@@ -7,12 +7,9 @@ use parent 'OIDC::Lite::Server::DataHandler';
 
 use String::Random;
 
-use OAuth::Lite2::Server::Error;
-use OIDC::Lite::Server::Error;
 use OIDC::Lite::Model::AuthInfo;
 use OAuth::Lite2::Model::AccessToken;
 use OIDC::Lite::Model::IDToken;
-use OIDC::Lite::Model::ClientInfo;
 
 my %ID_POD = (
     auth_info    => 0,
@@ -106,6 +103,12 @@ sub get_auth_info_by_refresh_token {
 
 sub get_auth_info_by_id {
     my ($self, $auth_id) = @_;
+
+    # for croak test
+    if ( $auth_id == 99 ) {
+        return q{invalid object};
+    }
+
     return $AUTH_INFO{$auth_id};
 }
 
@@ -113,6 +116,12 @@ sub get_auth_info_by_id {
 #   - device_token
 sub get_auth_info_by_code {
     my ($self, $device_code) = @_;
+
+    # for croak test
+    if ( $device_code eq q{code_invalid_croak} ) {
+        return q{invalid object};
+    }
+
     for my $id (keys %AUTH_INFO) {
         my $auth_info = $AUTH_INFO{$id};
         return $auth_info if ($auth_info->code && $auth_info->code eq $device_code);
@@ -136,9 +145,10 @@ sub create_or_update_auth_info {
                     unless($id_token);
     $code = sprintf q{code_%d}, $id
                     unless($code);
-    my @claims = (q{user_id}, q{email});
+    my @claims;
+    @claims = (q{user_id}, q{email}) if $scope;
 
-    my $auth_info = OIDC::Lite::Model::AuthInfo->new({
+    my %attrs = (
         id              => $id,
         client_id       => $client_id,
         user_id         => $user_id,
@@ -146,7 +156,15 @@ sub create_or_update_auth_info {
         refresh_token   => $refresh_token,
         id_token        => $id_token,
         userinfo_claims => \@claims,
-    });
+    );
+    # for attribute lacked case
+    if ( $code eq q{code_without_optional_attr} ) {
+        delete $attrs{scope};
+        delete $attrs{refresh_token};
+        delete $attrs{id_token};
+    }
+
+    my $auth_info = OIDC::Lite::Model::AuthInfo->new(\%attrs);
     $auth_info->code($code) if $code;
     $auth_info->redirect_uri($redirect_uri) if $redirect_uri;
 
@@ -163,6 +181,11 @@ sub create_or_update_access_token {
     my $auth_info = $params{auth_info};
     my $auth_id = $auth_info->id;
 
+    # for croak test
+    if ( $auth_info->code eq q{code_for_croak_at} ) {
+        return q{invalid object};
+    }
+
     my $id = ref($self)->gen_next_access_token_id();
     my $token = sprintf q{access_token_%d}, $id;
 
@@ -173,6 +196,12 @@ sub create_or_update_access_token {
         created_on => time(),
     );
 
+    # for attribute lacked case
+    if ( $auth_info->code eq q{code_without_optional_attr} or 
+         ($auth_info->scope && $auth_info->scope eq q{no_exp openid})) {
+        delete $attrs{expires_in};
+    }
+
     my $access_token = OAuth::Lite2::Model::AccessToken->new(\%attrs);
     $ACCESS_TOKEN{$auth_id} = $access_token;
     return $access_token;
@@ -180,6 +209,12 @@ sub create_or_update_access_token {
 
 sub get_access_token {
     my ($self, $token) = @_;
+
+    # for croak test
+    if ( $token eq q{token_for_croak} ) {
+        return q{invalid object};
+    }
+
     for my $auth_id ( keys %ACCESS_TOKEN ) {
         my $t = $ACCESS_TOKEN{ $auth_id };
         if ($t->token eq $token) {
@@ -248,12 +283,42 @@ sub validate_scope{
 
 sub validate_display{
     my ($self, $display) = @_;
-    return (!$display || $display ne "invalid");
+    return (!$display || $display ne "wap");
 }
 
 sub validate_prompt{
     my ($self, $prompt) = @_;
-    return (!$prompt || $prompt ne "invalid");
+    return (!$prompt || $prompt ne "none");
+}
+
+sub validate_max_age{
+    my ($self, $param) = @_;
+    return (!$param->{max_age} || $param->{max_age} > 0);
+}
+
+sub validate_ui_locales{
+    my ($self, $ui_locales) = @_;
+    return (!$ui_locales || $ui_locales ne "invalid");
+}
+
+sub validate_claims_locales{
+    my ($self, $claims_locales) = @_;
+    return (!$claims_locales || $claims_locales ne "invalid");
+}
+
+sub validate_id_token_hint{
+    my ($self, $param) = @_;
+    return (!$param->{id_token_hint} || $param->{id_token_hint} ne "invalid");    
+}
+
+sub validate_login_hint{
+    my ($self, $param) = @_;
+    return (!$param->{login_hint} || $param->{login_hint} ne "invalid");    
+}
+
+sub validate_acr_values{
+    my ($self, $param) = @_;
+    return (!$param->{acr_values} || $param->{acr_values} ne "invalid");    
 }
 
 sub validate_request{
@@ -266,11 +331,6 @@ sub validate_request_uri{
     return (!$param->{request_uri} || $param->{request_uri} ne "invalid");
 }
 
-sub validate_id_token{
-    my ($self, $id_token) = @_;
-    return (!$id_token || $id_token ne "invalid");
-}
-
 sub get_user_id_for_authorization {
     my ($self) = @_;
     return 1;
@@ -297,70 +357,6 @@ sub create_id_token {
     );
 }
 
-sub client_associate {
-    my ($self, $param, $access_token) = @_;
-   
-    # check access token 
-    OAuth::Lite2::Server::Error::InvalidToken->throw(
-    ) if($access_token and $access_token ne 'test_access_token');
- 
-    # check redirect_uri
-    OIDC::Lite::Server::Error::InvalidRedirectUri->throw(
-    ) if($param->{redirect_uris} ne 'http://example.org/redirect');
-
-    # configuration parameter
-    return if($param->{application_type} and $param->{application_type} eq 'invalid');
- 
-    # success
-    my $client_info = OIDC::Lite::Model::ClientInfo->new(
-        client_id => q{test_client_id},
-        client_secret => q{test_client_secret},
-        registration_access_token => q{test_registration_access_token},
-        expires_at => 1234567,
-        metadata => {
-            redirect_uris => $param->{redirect_uris},
-        }
-    );
-    return $client_info;
-}
-
-sub client_update {
-    my ($self, $param, $access_token) = @_;
-   
-    # check redirect_uri
-    OIDC::Lite::Server::Error::InvalidRedirectUri->throw(
-    ) if($param->{redirect_uris} ne 'http://example.org/redirect');
-
-    # configuration parameter
-    return if($param->{application_type} and $param->{application_type} eq 'invalid');
-
-    # success
-    my $client_info = OIDC::Lite::Model::ClientInfo->new(
-        client_id => q{test_client_id},
-        client_secret => q{test_client_secret},
-        registration_access_token => q{test_registration_access_token},
-        expires_at => 1234567,
-        metadata => {
-            redirect_uris => $param->{redirect_uris},
-        }
-    );
-    return $client_info;
-}
-
-sub rotate_secret {
-    my ($self, $access_token) = @_;
-   
-    # configuration parameter
-    return if(!$access_token or $access_token ne qw{test_access_token});
- 
-    # success
-    my $client_info = OIDC::Lite::Model::ClientInfo->new(
-        client_id => q{test_client_id},
-        client_secret => q{test_client_secret_rotate},
-        registration_access_token => q{test_registration_access_token_rotate},
-        expires_at => 1234567
-    );
-    return $client_info;
-}
+# for test
 
 1;
